#include "VehicleController.h"

VehicleController::VehicleController(CVehicle* v, Track* track)
: m_vehicle(v), m_keyStat{false}, mTrack(track)
{
}

VehicleController::~VehicleController()
{
}

void VehicleController::setVehicle(CVehicle* v)
{
    m_vehicle = v;
}

edge::Vehicle* VehicleController::getVehicle()
{
    return m_vehicle;
}

bool VehicleController::keyPressed( const OIS::KeyEvent &arg )
{
    m_keyStat[arg.key] = true;
    return true;
}

bool VehicleController::keyReleased( const OIS::KeyEvent &arg )
{
    m_keyStat[arg.key] = false;
    return true;
}

bool VehicleController::mouseMoved( const OIS::MouseEvent &arg )
{
    return true;
}

bool VehicleController::mousePressed( const OIS::MouseEvent &arg, OIS::MouseButtonID id )
{
    return true;
}

bool VehicleController::mouseReleased( const OIS::MouseEvent &arg, OIS::MouseButtonID id )
{
    return true;
}

void VehicleController::update()
{
    float nP2F = mTrack->getPointToFinish(edge::Util::toVector3(m_vehicle->getPosition()));
    float sgn1, sgn2;

    if(m_vehicle->mP2Finish >= 0)
        sgn1 = 1.f;
    else
        sgn1 = -1.f;
    if(nP2F >= 0)
        sgn2 = 1.f;
    else
        sgn2 = -1.f;

    if(sgn1 + sgn2 == 0 && mTrack->testDistance(edge::Util::toVector3(m_vehicle->getPosition())))
    {
        m_vehicle->mLap += sgn2;
    }

    m_vehicle->mP2Finish = nP2F;

    float speed = m_vehicle->getCurrentSpeedKmh()+1; // +1 to avoid division by 0
    float steering = m_vehicle->getSteeringValue(0);

    m_vehicle->applyEngineForce(0.f,0);
    m_vehicle->applyEngineForce(0.f,1);
    m_vehicle->setBrake(0.f,0);
    m_vehicle->setBrake(0.f,1);
    m_vehicle->setBrake(0.f,2);
    m_vehicle->setBrake(0.f,3);

    if(m_keyStat[OIS::KC_UP] && speed<m_vehicle->getMaxSpeed())
    {
        float force = m_vehicle->getSpeedInc() - (m_vehicle->getCurrentSpeedKmh() * m_vehicle->getSpeedInc() / m_vehicle->getMaxSpeed());
        m_vehicle->applyEngineForce(force,0);
        m_vehicle->applyEngineForce(force,1);
    }
    else if(m_keyStat[OIS::KC_DOWN])
    {
        if(speed<3 && speed>-60.f)
        {
            m_vehicle->applyEngineForce(-750.f,0);
            m_vehicle->applyEngineForce(-750.f,1);
        }
        else
        {
            m_vehicle->applyEngineForce(0.f,0);
            m_vehicle->applyEngineForce(0.f,1);
            m_vehicle->setBrake(m_vehicle->getBrakeVal(),2);
            m_vehicle->setBrake(m_vehicle->getBrakeVal(),3);
        }
    }
    else
    {
        m_vehicle->setBrake(1,0);
        m_vehicle->setBrake(1,1);
    }
    if(m_keyStat[OIS::KC_SPACE])
    {
        m_vehicle->setBrake(m_vehicle->getBrakeVal(),0);
        m_vehicle->setBrake(m_vehicle->getBrakeVal(),1);
    }

    float maxsteer;
    if(m_vehicle->getCurrentSpeedKmh() <= 0)
        maxsteer = m_vehicle->getMaxSteer();
    else
    maxsteer = (m_vehicle->getMaxSpeed() / (m_vehicle->getCurrentSpeedKmh())) / 11.f * (m_vehicle->getMaxSteer());
    if(maxsteer > m_vehicle->getMaxSteer())
        maxsteer = m_vehicle->getMaxSteer();
    /*printf("SPEED=%f\n", speed);
    printf("SCV=%f\n", scv);
    printf("MAX=%i\n", (int)(m_steerMax/scv*180.f/M_PI));*/
    //scv /= 2.f;
    if(m_keyStat[OIS::KC_LEFT] && steering<maxsteer)
    {
        m_vehicle->setSteeringValue(steering+m_vehicle->getSteerInc(),0);
        m_vehicle->setSteeringValue(steering+m_vehicle->getSteerInc(),1);
    }
    if(m_keyStat[OIS::KC_RIGHT] && steering>-maxsteer)
    {
        m_vehicle->setSteeringValue(steering-m_vehicle->getSteerInc(),0);
        m_vehicle->setSteeringValue(steering-m_vehicle->getSteerInc(),1);
    }
    if(!m_keyStat[OIS::KC_LEFT] && !m_keyStat[OIS::KC_RIGHT])
    {
        float zsv;
        if(steering>m_vehicle->getSteerInc()*5) zsv = -m_vehicle->getSteerInc()*5 + steering;
        else if(steering<-m_vehicle->getSteerInc()*5) zsv = m_vehicle->getSteerInc()*5 + steering;
        else zsv = 0.f;
        m_vehicle->setSteeringValue(zsv,0);
        m_vehicle->setSteeringValue(zsv,1);
    }
}
